#ifndef MAC_TASKS_H_
#define MAC_TASKS_H_

#include "FreeRTOS.h"
#include "task.h"
#include "Board.h" 
#include "udp.h"
#include "tcp.h"

/* adc_monitor_via_UDP is the FREERTOS task. It handles data transfers on the ethernet line.
 * Normally this task should be implemented(and in the beginning it was) using 
 * queues. However, in this project we have very quick response and a simple 
 * programming architecture due to overhead of ADC interrupts. So we implemented 
 * this task using a semaphore. This task simply sleeps on a semaphore. Semaphore is
 * released by the ADC interrupt function. When we take the interrupt, this task 
 * wakes up and sends the referenced buffer from ethernet.
 * 
 * This task also registers on a callback function. When a datagram is received from
 * the line, lwip calls this callback function. Then we answer to the datagram
 * accordingly using this callback.
 * 
 * Since this is the only ethernet task in this project, we register MAC interface
 * in this function. Then this functions establishes a connection on an UDP port on
 * this default interface.
 */ 
void adc_monitor_via_UDP( void *pvParameters );
void adc_monitor_via_TCP( void *pvParameters );

/* vlwIPInit function comes with FREERTOS demo projects. It initialises the 
 * low-level IP stack. This function should be called before any action on the ethernet
 * interface.
 */
void vlwIPInit( void );

/* datagram_recved is the callback function registered by the v_pc_comm task.
 * When a datagram is received on the registered port this callback function is
 * called by the lwIP stack. 
 * 
 * pcb is the udp_pcb structure which is used to respond to the message.
 * p is the pbuf buffer containing the message contents which should be cleared
 * before exiting this function.
 * Other parameters are not used but needed since lwIP calls this callback.
 */
inline void datagram_recved(void *arg, struct udp_pcb *pcb, struct pbuf *p, struct ip_addr *addr, u16_t port);

/* send_block function is used to send a memory block from the line. This function
 * internally allocates a buffer for the message and copies data from *block to this
 * buffer. length is the number memory locations to send, NOT the number of bytes
 * to send. Data is send using pcb structure.
 */
inline void send_block(int *block , short length , struct udp_pcb *pcb);
inline void send_tcp_block(int *block , short length , struct tcp_pcb *pcb);

/* send_message function is smilar to the send_block function but using chars.
 * It is specially for sending text messages from the ethernet line. Working principle
 * is same with send_block except that length argument can be treated as message
 * length in bytes.
 */
inline void send_message(char *mes , struct udp_pcb *pcb);
inline void send_tcp_message(char *mes , struct tcp_pcb *pcb);

/* These function are used by the callback function,datagram_recved.
 * Actually these function were included in the callbck function. Later, they are
 * seperated from the callback for simple modularity and simplicity. That's why 
 * they are inlined. 
 */ 
inline void read_gain( char *mes );
inline void write_gain( char *mes , int *settings );
inline char offset_correct( int *settings );
inline void configure_device( int *settings , struct udp_pcb *pcb, struct pbuf *p );
inline void configure_device_tcp( int *settings , struct tcp_pcb *pcb, struct pbuf *p );
inline void process_message( int *settings , struct udp_pcb *pcb, struct pbuf *p );
inline void process_tcp_message( int *settings , struct tcp_pcb *pcb, struct pbuf *p );	

err_t netio_accept(void *arg, struct tcp_pcb *pcb, err_t err);
struct tcp_pcb* netio_init(void);
err_t netio_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err);
err_t netio_sent(void *arg, struct tcp_pcb *tpcb, u16_t len);

#endif /*MAC_TASKS_H_*/
